/* CPU Functions */
const int ZF=0x40;
const int SF=0x80;
int F=0x40;       // ZF set

void SetFlag(int *Flags, int FlagMask)
{
  *Flags=(*Flags)|FlagMask;
}

bool FlagIsSet(int Flags, int FlagMask)
{
  return ((Flags&FlagMask)!=0);
}

void Test(int Ax)
{
  F=0;
  if (Ax<0) {
    SetFlag(&F, SF);
  } else if (Ax==0) {
    SetFlag(&F, ZF);
  }
}

void push(int Ax)
{
  SP=SP-1;
  Memory[SP]=Ax;
}

int pop(void)
{
  int Data;

  Data=Memory[SP];
  SP=SP+1;
  return Data;
}

bool Interpret(int *IP)
{
  static int Ax=0;
  static int Bx=0;
  bool running=true;
  int operand;
  int IR;

  //Fetch Instruction
  IR=Memory[*IP];
  *IP=*IP+1;
  if (IR==_movAxVar) {
    // Fetch Address
    operand=Memory[*IP];
    *IP=*IP+1;
    Ax=Memory[operand];
  } else if (IR==_movVarAx) {
    // Fetch Address
    operand=Memory[*IP];
    *IP=*IP+1;
    Memory[operand]=Ax;
  } else if (IR==_movAxImm) {
    // Fetch Address
    operand=Memory[*IP];
    *IP=*IP+1;
    Ax=operand;
  } else if (IR==_movAxBx) {
    Ax=Bx;
  } else if (IR==_movBxAx) {
    Bx=Ax;
  } else if (IR==_pushAx) {
    push(Ax);
  } else if (IR==_popAx) {
    Ax=pop();
  } else if (IR==_popBx) {
    Bx=pop();
  } else if (IR==_addBx) {
    Ax=Ax+Bx;
  } else if (IR==_subBx) {
    Ax=Ax-Bx;
  } else if (IR==_mulBx) {
    Ax=Ax*Bx;
  } else if (IR==_divBx) {
    Ax=Ax/Bx;
  } else if (IR==_jmp) {
    // Fetch Address
    operand=Memory[*IP];
    *IP=operand;
  } else if (IR==_jz) {
    if (FlagIsSet(F, ZF)) {
    // Fetch Address
    operand=Memory[*IP];
      *IP=operand;
    } else {
      *IP=*IP+1;
    }
  } else if (IR==_orAxAx) {
    Test(Ax);
  } else if (IR==_xorAxAx) {
    Ax=0;
    Test(Ax);
  } else if (IR==_cmpBx) {
    Test(Ax-Bx);
  } else if (IR==_setEq) {
    Ax=0;
    if (FlagIsSet(F, ZF)) Ax=1;
  } else if (IR==_setNE) {
    Ax=0;
    if (!FlagIsSet(F, ZF)) Ax=1;
  } else if (IR==_setLT) {
    Ax=0;
    if (FlagIsSet(F, SF)) Ax=1;
  } else if (IR==_setLE) {
    Ax=0;
    if (FlagIsSet(F, SF)||FlagIsSet(F, ZF)) Ax=1;
  } else if (IR==_setGT) {
    Ax=1;
    if (FlagIsSet(F, SF)||FlagIsSet(F, ZF)) Ax=0;
  } else if (IR==_setGE) {
    Ax=1;
    if (FlagIsSet(F, SF)) Ax=0;
  } else if (IR==_wrtAx) {
    printf("%d ",Ax);
  } else if (IR==_wrtLn) {
    printf("\n");
  } else if (IR==_rdAx) {
    // To do!
  } else if (IR==_halt) {
    running=false;
  } else {
    // Error
    printf("Simple Interpreter Error: Unknown Command %d\n",IR);
    exit(1);
  }
  return running;
}

void SimpleInt(void)
{
  int IP=0;

  printf("Simple Interpreter:\n");
  while (Interpret(&IP));
}
